@@ -1,7 +1,7 @@
 /* gxmessage - an xmessage clone using GTK2
  *
  * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2012, 2015 Timothy Richard Musson
+ * 2012, 2014 Timothy Richard Musson
  *
  * Email: Tim Musson <trmusson@gmail.com>
  * WWW:   http://homepages.ihug.co.nz/~trmusson/programs.html#gxmessage
@@ -32,15 +32,13 @@
 
 /* Details for Copyright and bug report messages: */
 #define AUTHOR  "Timothy Richard Musson"
-#define YEAR    "2015"
+#define YEAR    "2014"
 #define MAILTO  "<trmusson@gmail.com>"
 
 /* If the following file exists, the -noescape option will be allowed to work: */
 #define ALLOW_NOESCAPE PACKAGE_DATA_DIR \
             G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "allow_noescape"
 
-#define MAX_WINDOW_SIZE (G_MAXUINT16 / 2)
-
 #ifdef ENABLE_NLS
 #  include <libintl.h>
 #  define _(String) gettext(String)
@@ -359,7 +357,7 @@ cb_window_destroy (GtkWidget *widget, GdkEvent *event, gpointer data)
 gboolean
 cb_key_press (GtkWidget *w, GdkEventKey *event, gpointer data)
 {
-	if (gx.allow_escape && (event->keyval == GDK_Escape)) {
+	if (gx.allow_escape && (event->keyval == GDK_KEY_Escape)) {
 		gtk_main_quit ();
 	}
 	return FALSE;
@@ -403,25 +401,45 @@ cb_timeout (gint *timeout)
 }
 
 
-gboolean
-label_width_okay (const gchar *str)
+gint
+geometry_get_number (const gchar **geometry)
 {
-	GtkWidget *dummy;
-	PangoLayout *layout;
-	int width;
+	/* (This function is taken from GNOME's gnome-geometry.c) */
 
-	dummy = gtk_label_new (NULL);
-	layout = gtk_label_get_layout (GTK_LABEL(dummy));
-	pango_layout_set_text (layout, str, -1);
-	pango_layout_get_pixel_size (layout, &width, NULL);
-	gtk_widget_destroy (dummy);
+	gint value = 0;
+	gint mult  = 1;
 
-	if (width > MAX_WINDOW_SIZE) {
-		g_warning ("%s: button too wide\n", PACKAGE);
-		return FALSE;
+	if (**geometry == '-') {
+		mult = -1;
+		(*geometry)++;
 	}
+	while ( **geometry && isdigit(**geometry) ) {
+		value = value * 10 + (**geometry - '0');
+		(*geometry)++;
+	}
+	return value * mult;
+}
 
-	return TRUE;
+
+void
+geometry_parse_string (const gchar *geometry, gint *width, gint *height)
+{
+	/* Parse geometry string into width and height */
+
+	*width = *height = -1;
+
+	if (*geometry == '=') geometry++;
+	if (*geometry == '\0') return;
+
+	*width = geometry_get_number (&geometry);
+	if (*width < -1) *width = -1;
+
+	if (*geometry == 'x') {
+		geometry++;
+		if (*geometry == '\0') return;
+		*height = geometry_get_number (&geometry);
+		if (*height < -1) *height = -1;
+	}
 }
 
 
@@ -478,10 +496,10 @@ window_create (void)
 	/* window contents */
 	gtk_container_set_border_width (GTK_CONTAINER(window), 12);
 
-	vbox = gtk_vbox_new (FALSE, 12);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
 	gtk_container_add (GTK_CONTAINER(window), vbox);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
+	vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_box_pack_start (GTK_BOX(vbox), vbox2, TRUE, TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER(vbox2), 0);
 
@@ -547,7 +565,7 @@ window_create (void)
 
 		button = button_first (gx.button_list);
 
-		btn_box = gtk_hbutton_box_new ();
+		btn_box = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
 		gtk_button_box_set_layout (GTK_BUTTON_BOX(btn_box),
 		                           GTK_BUTTONBOX_END);
 		gtk_box_set_spacing (GTK_BOX(btn_box), 6);
@@ -568,22 +586,12 @@ window_create (void)
 					s++;
 				}
 				s = g_strconcat ("gtk-", p, NULL);
-				if (!label_width_okay (s)) {
-					/* XXX: Truncate and carry on, or quit? */
-					prog_cleanup ();
-				}
 				btn = gtk_button_new_from_stock (s);
 				g_free (s);
 				g_free (p);
 			}
 			else {
-
-				if (!label_width_okay (button->label)) {
-					/* XXX: Truncate and carry on, or quit? */
-					prog_cleanup ();
-				}
 				btn = gtk_button_new_with_mnemonic (button->label);
-
 			}
 
 			g_signal_connect (G_OBJECT(btn), "clicked",
@@ -616,8 +624,7 @@ window_create (void)
 		gtk_widget_modify_font (dummy,
 		    gtk_widget_get_style (message_widget)->font_desc);
 		gtk_container_add (GTK_CONTAINER(vbox), dummy);
-		gtk_widget_show (dummy);
-		gtk_widget_size_request (dummy, &size_req);
+		gtk_widget_get_preferred_size (dummy, &size_req, NULL);
 		gtk_widget_destroy (dummy);
 		/* ~50 pixels for borders and scrollbar space */
 		win_w = size_req.width + 50;
@@ -627,29 +634,31 @@ window_create (void)
 	}
 
 	if (gx.entry_str != NULL) {
-		gtk_widget_size_request (gx.entry_widget, &size_req);
+		gtk_widget_get_preferred_size (gx.entry_widget, &size_req, NULL);
 		win_h = win_h + size_req.height + 12;
 	}
 
 	if (gx.do_buttons) {
-		gtk_widget_size_request (btn, &size_req);
+		gtk_widget_get_preferred_size (btn, &size_req, NULL);
 		win_h = win_h + size_req.height + 12;
 	}
 
-	gtk_window_set_position (GTK_WINDOW(window), gx.window_position);
-	gtk_window_set_default_size (GTK_WINDOW(window), win_w, win_h);
-
 	if (gx.geom_str != NULL) {
-		gtk_widget_show_all (vbox); /* must precede parse_geometry */
-		gtk_window_parse_geometry (GTK_WINDOW(window), gx.geom_str);
+		gint tmp_w, tmp_h;
+		geometry_parse_string (gx.geom_str, &tmp_w, &tmp_h);
+		if (tmp_w != -1) win_w = tmp_w;
+		if (tmp_h != -1) win_h = tmp_h;
 	}
 
+	gtk_window_set_default_size (GTK_WINDOW(window), win_w, win_h);
+	gtk_window_set_position (GTK_WINDOW(window), gx.window_position);
+
 	/* open the window */
 	gtk_widget_show_all (window);
 
 	/* begin timeout */
 	if (gx.timeout != 0) {
-		gx.timeout_id = g_timeout_add (1000, (GtkFunction)cb_timeout,
+		gx.timeout_id = g_timeout_add (1000, (GSourceFunc)cb_timeout,
 		                               &gx.timeout);
 	}
 }
@@ -674,7 +683,7 @@ read_stdin (void)
 
 
 gchar*
-str_to_utf8 (const gchar *str)
+message_to_utf8 (const gchar *str)
 {
 	gchar *result;
 	GError *error = NULL;
@@ -806,7 +815,6 @@ main (gint argc, gchar *argv[])
 	GString *gstr = NULL;
 	gchar *ch = NULL, *tmpstr;
 	const gchar *fname = NULL;
-	const gchar *bstr = NULL;
 	gint opt, arg = 1;
 	gboolean ok;
 	gchar bu_default[] = "okay:0";
@@ -994,12 +1002,12 @@ main (gint argc, gchar *argv[])
 			g_printerr (_("%s: unable to read file\n"), PACKAGE);
 			prog_cleanup ();
 		}
-		gx.message_text = str_to_utf8 (tmpstr);
+		gx.message_text = message_to_utf8 (tmpstr);
 		gx.message_len = strlen (tmpstr);
 		g_free (tmpstr);
 	}
 	else if (gstr != NULL) {
-		gx.message_text = str_to_utf8 (gstr->str);
+		gx.message_text = message_to_utf8 (gstr->str);
 		gx.message_len = gstr->len;
 		g_string_free (gstr, TRUE);
 	}
